package com.hzya.frame.plugin.lets.plugin.outsourc;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.beanutil.BeanUtil;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.dao.*;
import com.hzya.frame.plugin.lets.entity.*;
import com.hzya.frame.plugin.lets.ofsvo.QueryOfsSoSaleOutVo;
import com.hzya.frame.plugin.lets.u8cdto.*;
import com.hzya.frame.plugin.lets.util.OfsStandardUtil;
import com.hzya.frame.plugin.lets.util.OnlyImplementProxyOrderUtil;
import com.hzya.frame.plugin.lets.util.QueryBdBusitypeUtil;
import com.hzya.frame.plugin.lets.util.RdclUtil;
import com.hzya.frame.ttxofs.dto.ofspoorder.OfsPoOrderData;
import com.hzya.frame.ttxofs.dto.ofspoorder.OfsPoOrderDetails;
import com.hzya.frame.ttxofs.dto.ofspoorder.OfsPoOrderHeader;
import com.hzya.frame.ttxofs.dto.stock.StockinOrderSearchResponse;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.util.*;

/**
 * O采购入库 -> U8C委外加工入库
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.plugin.outsourc
 * @Project：kangarooDataCenterV3
 * @name：ConsignmachiningIn
 * @Date：2024年8月26日 10:23:20
 * @Filename：ConsignmachiningIn
 */
public class ConsignmachiningIn extends PluginBaseEntity {

    Logger logger = LoggerFactory.getLogger(ConsignmachiningIn.class);

    @Autowired
    private QueryBdBusitypeUtil queryBdBusitypeUtil;

    @Autowired
    private IBdCorpDao iBdCorpDao;

    @Autowired
    private IBdInvmandocDao iBdInvmandocDao;

    @Autowired
    private IBdCalbodyDao iBdCalbodyDao;

    @Autowired
    private IBdStordocDao iBdStordocDao;

    @Autowired
    private RdclUtil rdclUtil;

    @Autowired
    private OfsStandardUtil ofsStandardUtil;

    @Autowired
    private IScOrderDao iScOrderDao;

    @Autowired
    private IScOrderBDao iScOrderBDao;

    @Autowired
    private OnlyImplementProxyOrderUtil onlyImplementProxyOrder;

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "com.hzya.frame.plugin.lets.plugin.outsourc.ConsignmachiningIn";
    }

    @Override
    public String getPluginName() {
        return "OFS采购入库单生成U8C委外加工入库(委外采购入库)";
    }

    @Override
    public String getPluginLabel() {
        return "OFS采购入库单生成U8C委外加工入库(委外采购入库)";
    }

    @Override
    public String getPluginType() {
        return "3";
    }

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        return null;
    }

    /**
     * 根据单号查询
     *
     * @author liuyang
     */
    public void startImplement(String startTime, String endTime) {
        String threadNameStrStart = StrUtil.format("开始（委外采购）OFS采购入库单生成U8C委外加工入库 开始时间：{} 结束时间：{}", startTime, endTime);
        logger.info(threadNameStrStart);

        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoArrayList = new ArrayList<>();
                    QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
                    queryOfsSoSaleOutVo.setClosedAt_start(startTime);
                    queryOfsSoSaleOutVo.setClosedAt_end(endTime);
                    queryOfsSoSaleOutVo.setClientCode("LETS");
                    queryOfsSoSaleOutVo.setPageNo(1L);
                    queryOfsSoSaleOutVo.setPageSize(50L);
                    queryOfsSoSaleOutVo.setInternalInstructionType("PURCHASE");
                    queryOfsSoSaleOutVo.setStatus(900L);
                    ofsStandardUtil.queryOfsReturnGoods(queryOfsSoSaleOutVo, returnGoodHeaderDetailsDataDtoArrayList, 1L, "ofs.receipt.search");
                    logger.info("数据返回行数：{}", returnGoodHeaderDetailsDataDtoArrayList.size());
                    if (returnGoodHeaderDetailsDataDtoArrayList.size() > 0) {
                        getSet(returnGoodHeaderDetailsDataDtoArrayList);
                    } else {
                        logger.info("没有查询到任何数据！不需要同步");
                    }
                } catch (Exception e) {
                    logger.error("startImplement方法抛出异常", e);
                }
            }
        }, threadNameStrStart);
        thread.start();
        try {
            thread.join();
        } catch (Exception e) {
            logger.error("thread.join()异常", e);
        }
        String threadNameStrEnd = StrUtil.format("结束（委外采购）OFS采购入库单生成U8C委外加工入库 开始时间：{} 结束时间：{}", startTime, endTime);
        logger.info(threadNameStrEnd);
    }

    /**
     * 根据单号查询
     *
     * @author liuyang
     */
    public void startImplement(String code) {
        String threadNameStrStart = StrUtil.format("开始（委外采购）OFS采购入库单生成U8C委外加工入库 OFS采购入库单号：{}", code);
        logger.info(threadNameStrStart);

        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoArrayList = new ArrayList<>();
                    QueryOfsSoSaleOutVo queryOfsSoSaleOutVo = new QueryOfsSoSaleOutVo();
                    queryOfsSoSaleOutVo.setClientCode("LETS");
                    queryOfsSoSaleOutVo.setPageNo(1L);
                    queryOfsSoSaleOutVo.setPageSize(50L);
                    queryOfsSoSaleOutVo.setCode(code);
                    queryOfsSoSaleOutVo.setInternalInstructionType("PURCHASE");
                    queryOfsSoSaleOutVo.setStatus(900L);
                    ofsStandardUtil.queryOfsReturnGoods(queryOfsSoSaleOutVo, returnGoodHeaderDetailsDataDtoArrayList, 1L, "ofs.receipt.search");
                    logger.info("数据返回行数：{}", returnGoodHeaderDetailsDataDtoArrayList.size());
                    if (returnGoodHeaderDetailsDataDtoArrayList.size() > 0) {
                        getSet(returnGoodHeaderDetailsDataDtoArrayList);
                    } else {
                        logger.info("没有查询到任何数据！不需要同步");
                    }
                } catch (Exception e) {
                    logger.error("startImplement方法抛出异常", e);
                }
            }
        }, threadNameStrStart);
        thread.start();
        try {
            thread.join();
        } catch (Exception e) {
            logger.error("thread.join()异常", e);
        }
        String threadNameStrEnd = StrUtil.format("结束（委外采购）OFS采购入库单生成U8C委外加工入库 OFS采购入库单号：{}", code);
        logger.info(threadNameStrEnd);
    }

    /**
     * 环境预配置
     *
     * @param returnGoodHeaderDetailsDataDtoArrayList 采购入库单
     * @author liuyang
     */
    private void getSet(List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoArrayList) throws Exception {
        //过滤成功的数据
        List<StockinOrderSearchResponse.StockinOrder> stockinOrderList = filterData(returnGoodHeaderDetailsDataDtoArrayList);
        //执行推送主逻辑
        implement(stockinOrderList);
    }

    /**
     * 过滤掉成功的数据
     *
     * @author liuyang
     */
    private List<StockinOrderSearchResponse.StockinOrder> filterData(List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoArrayList) {
        List<StockinOrderSearchResponse.StockinOrder> headerDetailsDtoList1 = new ArrayList<>();
        if (returnGoodHeaderDetailsDataDtoArrayList != null && returnGoodHeaderDetailsDataDtoArrayList.size() > 0) {
            //TODO 出库单明细主键，需要O返回，目前没有，已经提需求
            headerDetailsDtoList1.addAll(returnGoodHeaderDetailsDataDtoArrayList);
        }
        return headerDetailsDtoList1;
    }

    /**
     * 执行主逻辑
     *
     * @author liuyang
     */
    private void implement(List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoList) throws Exception {
        if (returnGoodHeaderDetailsDataDtoList != null && returnGoodHeaderDetailsDataDtoList.size() > 0) {
            try {
                // 查询基本档案
                List<ConsignmachiningInSonDto> consignmachiningInSonDtos = queryBasicArchives(returnGoodHeaderDetailsDataDtoList);
                // 查询采购收发类别
                BdRdclEntity bdRdclEntity = rdclUtil.queryRdClObject("105");

                if (consignmachiningInSonDtos != null && consignmachiningInSonDtos.size() > 0) {
                    for (int i = 0; i < consignmachiningInSonDtos.size(); i++) {
                        ConsignmachiningInSonDto consignmachiningInSonDto = consignmachiningInSonDtos.get(i);
//                        StockinOrderSearchResponse.StockinOrder.StockinH header = consignmachiningInSonDto.getHeader();
                        List<StockinOrderSearchResponse.StockinOrder.StockinB> details = consignmachiningInSonDto.getDetails();

                        String generateBusinessDate = createGenerateBusinessDate(consignmachiningInSonDto); //生成业务日期
                        OfsPoOrderData ofsPoOrderData = consignmachiningInSonDto.getOfsPoOrderData();//OFS采购订单
                        ScOrderEntity scOrderEntity = consignmachiningInSonDto.getScOrderEntity();//U8C委外订单
                        BdCorpEntity bdCorpEntity = consignmachiningInSonDto.getBdCorpEntity();//公司
                        BdCalbodyEntity bdCalbodyEntity = consignmachiningInSonDto.getBdCalbodyEntity();//库存组织
                        BdStordocEntity bdStordocEntity = consignmachiningInSonDto.getBdStordocEntity();//仓库
//                        StockinOrderSearchResponse.StockinOrder.StockinH header1 = consignmachiningInSonDto.getHeader();
                        List<StockinOrderSearchResponse.StockinOrder.StockinB> details1 = consignmachiningInSonDto.getDetails();//OFS采购入库单
                        List<ScOrderBEntity> scOrderBEntityList = consignmachiningInSonDto.getScOrderBEntityList();//U8C委外订单明细行主键

                        OfsPoOrderHeader ofsPoOrderDataHeader = ofsPoOrderData.getHeader();
                        List<OfsPoOrderDetails> ofsPoOrderDataDetails = ofsPoOrderData.getDetails();

                        // 代理委外入库业务流程
                        BdBusitypeEntity bdBusitypeEntity = u8cOperationFlow(bdCorpEntity.getPkCorp());

                        //委外入库表头
                        ConsignmachiningInHeadDto poOrderParentDto = new ConsignmachiningInHeadDto();
                        poOrderParentDto.setPk_corp(bdCorpEntity.getPkCorp());
                        poOrderParentDto.setCoperatorid("0001A110000000000U3D");
                        poOrderParentDto.setCwarehouseid(bdStordocEntity.getPkStordoc());
                        poOrderParentDto.setPk_calbody(bdCalbodyEntity.getPkCalbody());
                        poOrderParentDto.setCbiztype(bdBusitypeEntity.getPkBusitype());
                        poOrderParentDto.setCdispatcherid(bdRdclEntity.getPkRdcl());//收发类别
                        poOrderParentDto.setDbilldate(generateBusinessDate);//入库日期

                        //委外入库表头明细行
                        List<ConsignmachiningInBodyDto> consignmachiningInBodyDtoArrayList = new ArrayList<>();
                        for (int j = 0; j < details.size(); j++) {
                            StockinOrderSearchResponse.StockinOrder.StockinB stockinB = details.get(j);
                            //匹配U8C采购订单明细行
                            ScOrderBEntity u8cPoOrderDetail = findOfsPoOrderDetailAndU8cPoOrderDetail(stockinB, ofsPoOrderData, scOrderBEntityList);
                            //通过O采购订单入库单明细行，联查OFS采购订单明细行
                            OfsPoOrderDetails ofsPoOrderDetailRowTraget = findOfsPoOrderDetailRowTraget(stockinB, ofsPoOrderData);
                            //查询存货管理档案
                            BdInvmandocEntity bdInvmandocEntity = queryInventoryMan(stockinB, bdCorpEntity.getPkCorp());

                            //计算含税单价
                            BigDecimal noriginalcurprice = null;
                            String receivedQty = stockinB.getReceivedQty();//实收数量
                            String fulfillAmount = ofsPoOrderDetailRowTraget.getFulfillAmount();//实收累计总金额
                            try {
                                noriginalcurprice = new BigDecimal(fulfillAmount).divide(new BigDecimal(receivedQty), 20, BigDecimal.ROUND_HALF_UP).setScale(4, BigDecimal.ROUND_HALF_UP);
                            } catch (Exception e) {
                                logger.error("计算采购入库单原币含税单价失败", e);
                                Assert.state(false, "计算采购入库单原币含税单价失败 原因：{}", e.getMessage());
                            }

                            ConsignmachiningInBodyDto consignmachiningInBodyDto = new ConsignmachiningInBodyDto();
                            consignmachiningInBodyDto.setCinventoryid(bdInvmandocEntity.getPkInvmandoc());//存货
                            //TODO 测试
                            consignmachiningInBodyDto.setNinnum("1");
//                            consignmachiningInBodyDto.setNinnum(new BigDecimal(stockinB.getReceivedQty()).stripTrailingZeros().toPlainString());//实收数量
                            consignmachiningInBodyDto.setNprice(noriginalcurprice.stripTrailingZeros().toPlainString());//单价（含税单价）
                            consignmachiningInBodyDto.setCsourcetype("61");//来源单据类型
                            consignmachiningInBodyDto.setVsourcerowno(u8cPoOrderDetail.getCrowno());//来源单据行号
                            consignmachiningInBodyDto.setVsourcebillcode(scOrderEntity.getVordercode());//来源单据号
                            consignmachiningInBodyDto.setWriteofftype("按备料发料核销");//核销方式
                            consignmachiningInBodyDtoArrayList.add(consignmachiningInBodyDto);
                        }

                        List<ConsignmachiningInHeadBodyDto> consignmachiningInHeadBodyDtoArrayList = new ArrayList<>();

                        ConsignmachiningInHeadBodyDto consignmachiningInHeadBodyDto = new ConsignmachiningInHeadBodyDto();
                        consignmachiningInHeadBodyDto.setParentvo(poOrderParentDto);
                        consignmachiningInHeadBodyDto.setChildrenvo(consignmachiningInBodyDtoArrayList);
                        consignmachiningInHeadBodyDtoArrayList.add(consignmachiningInHeadBodyDto);

                        Map<String, List<ConsignmachiningInHeadBodyDto>> stringStringMap = new HashMap<>();
                        stringStringMap.put("GeneralBillVO", consignmachiningInHeadBodyDtoArrayList);
                        //TODO 2024年8月26日 14:41:46 委外入库核销方式传不了！已经提交总部
                        //TODO 2024年8月26日 14:42:19 无法超委外订单数量入库！已经提交给总部
                        IcGeneralHResultDto icGeneralHResultDto = sendU8cConsignmachiningIn(JSON.toJSONString(stringStringMap));

                        String cgeneralhid = null;
                        String vbillcode = null;
                        IcGeneralHResultHeadDto parentvo = icGeneralHResultDto.getParentvo();
                        List<IcGeneralHResultBodyDto> childrenvo = icGeneralHResultDto.getChildrenvo();
                        if (parentvo != null) {
                            cgeneralhid = parentvo.getCgeneralhid();
                        }
                        if (childrenvo != null) {
                            vbillcode = parentvo.getVbillcode();
                        }
                        logger.info("推送U8C委外入库成功！委外入库单主键：{} 委外入库单编码：{}", cgeneralhid, vbillcode);
                    }
                } else {
                    logger.info("poOrderSonDto.size为0！");
                }
            } catch (Exception e) {
                logger.error("implement方法抛出异常", e);
            }
        }
    }

    /**
     * 档案转换(OFS->U8C)
     * 2024年8月23日 12:02:47 和妮姐沟通，单据流程U8C成品委外订单 -> 备料计划 -> U8C委外发料(核销)
     *
     * @author liuyang
     */
    private List<ConsignmachiningInSonDto> queryBasicArchives(List<StockinOrderSearchResponse.StockinOrder> returnGoodHeaderDetailsDataDtoList) {
        List<ConsignmachiningInSonDto> consignmachiningInSonDtoArrayList = new ArrayList<>();

        if (returnGoodHeaderDetailsDataDtoList != null && returnGoodHeaderDetailsDataDtoList.size() > 0) {
            try {
                for (int i = 0; i < returnGoodHeaderDetailsDataDtoList.size(); i++) {
                    StockinOrderSearchResponse.StockinOrder stockinOrder = returnGoodHeaderDetailsDataDtoList.get(i);
                    StockinOrderSearchResponse.StockinOrder.StockinH header = stockinOrder.getHeader();
                    List<StockinOrderSearchResponse.StockinOrder.StockinB> details = stockinOrder.getDetails();

                    //2024年8月25日 09:54:31 查询OFS采购订单
                    OfsPoOrderData ofsPoOrderData = ofsStandardUtil.queryOfsPoOrder(header.getRefOrderCode());
                    Assert.notNull(ofsPoOrderData, "无法查询到委外订单(OFS采购入库单->OSF采购订单->OFS采购订单主键->U8C委外订单自定义项20)");
                    OfsPoOrderHeader ofsPoOrderDataHeader = ofsPoOrderData.getHeader();
                    List<OfsPoOrderDetails> ofsPoOrderDataDetails = ofsPoOrderData.getDetails();

                    //如果该O采购订单对应的类型为：委外加工，则继续代码逻辑，否则就跳过代码逻辑
                    boolean checkResult = onlyImplementProxyOrder.onlyImplementProxyOrder(ofsPoOrderData, "WWJG");
                    if (!checkResult) {
                        continue;
                    }

                    //2024年8月25日 09:42:58 查询委外订单，如果这一步报错，后续的逻辑就没有必要执行了
                    String ofsId = ofsPoOrderDataHeader.getId();
                    Assert.notNull(ofsId, "通过接口查询到OFS采购订单，但是没有采购订单主键，业务逻辑无法完成");
                    ScOrderEntity scOrderEntity = queryScOrder(ofsId);

                    //2024年8月25日 14:15:45 根据委外订单id，查询委外订单表体明细行
                    List<ScOrderBEntity> scOrderBEntityList = queryScOrderDetail(scOrderEntity.getCorderid());

                    //2024年8月25日 10:09:20 委外入库公司，取委外订单上的公司，已经和妮姐确认，没有必要通过O的公司转换，因为不存在跨组织的问题
                    Assert.notNull(scOrderEntity.getPkCorp(), "O表头公司不能为空");
                    BdCorpEntity bdCorpEntity = new BdCorpEntity();
                    bdCorpEntity.setDr(0);
                    bdCorpEntity.setDataSourceCode("lets_u8c");
                    bdCorpEntity.setPkCorp(scOrderEntity.getPkCorp());
                    List<BdCorpEntity> bdCorpEntityList = iBdCorpDao.query(bdCorpEntity);
                    if (bdCorpEntityList.size() == 0) {
                        Assert.state(false, "根据委外订单公司主键:{}，无法匹配到委外入库单", scOrderEntity.getPkCorp());
                    } else if (bdCorpEntityList.size() >= 2) {
                        Assert.state(false, "根据委外订单公司主键:{}，匹配到{}个委外入库单", scOrderEntity.getPkCorp(), bdCorpEntityList.size());
                    }

                    // 2024年8月20日 23:51:56 收货库存组织，应该是通过收货公司查询得出收货库存组织，目前收货公司和采购公司保持一致，那么就以采购公司带出！这个逻辑已经和佳妮确认
                    BdCalbodyEntity bdCalbodyEntity = new BdCalbodyEntity();
                    bdCalbodyEntity.setDr(0);
                    bdCalbodyEntity.setDataSourceCode("lets_u8c");
                    bdCalbodyEntity.setPkCorp(bdCorpEntityList.get(0).getPkCorp());
                    List<BdCalbodyEntity> bdCalbodyEntities = iBdCalbodyDao.query(bdCalbodyEntity);
                    if (bdCalbodyEntities.size() == 0) {
                        Assert.state(false, "根据U8C采购公司{}，无法匹配到U8C发货库存组织", bdCorpEntityList.get(0).getPkCorp());
                    } else if (bdCalbodyEntities.size() >= 2) {
                        Assert.state(false, "根据U8C采购公司{}，匹配到U8C发货库存组织{}个", bdCorpEntityList.get(0).getPkCorp(), bdCalbodyEntities.size());
                    }

                    //入库仓库
                    String facilityCode = header.getFacilityCode();
                    Assert.notNull(facilityCode, "O售后入库单仓库facilityCode字段不能为空");
                    BdStordocEntity bdStordocEntity = new BdStordocEntity();
                    bdStordocEntity.setDr(0L);
                    bdStordocEntity.setDataSourceCode("lets_u8c");
                    bdStordocEntity.setPkCalbody(bdCalbodyEntities.get(0).getPkCalbody());
                    bdStordocEntity.setStorcode(facilityCode);
                    List<BdStordocEntity> bdStordocEntityList = iBdStordocDao.query(bdStordocEntity);
                    if (bdStordocEntityList.size() == 0) {
                        Assert.state(false, "根据O仓库编码+U8C收货库存组织主键，无法匹配到U8C仓库", facilityCode, bdCalbodyEntities.get(0).getPkCalbody());
                    } else if (bdStordocEntityList.size() >= 2) {
                        Assert.state(false, "根据O仓库编码+U8C收货库存组织主键，匹配到U8C仓库多个", facilityCode, bdCalbodyEntities.get(0).getPkCalbody());
                    }

                    ConsignmachiningInSonDto consignmachiningInSonDto = new ConsignmachiningInSonDto();
                    consignmachiningInSonDto.setOfsPoOrderData(ofsPoOrderData);
                    consignmachiningInSonDto.setScOrderEntity(scOrderEntity);
                    consignmachiningInSonDto.setBdCorpEntity(bdCorpEntityList.get(0));
                    consignmachiningInSonDto.setBdCalbodyEntity(bdCalbodyEntities.get(0));
                    consignmachiningInSonDto.setBdStordocEntity(bdStordocEntityList.get(0));
//                    consignmachiningInSonDto.setHeader(header);
                    consignmachiningInSonDto.setScOrderBEntityList(scOrderBEntityList);
                    consignmachiningInSonDto.setDetails(details);

                    BeanUtil.copyPropertiesV2(header, consignmachiningInSonDto);
                    consignmachiningInSonDtoArrayList.add(consignmachiningInSonDto);
                }
                //成功
            } catch (Exception e) {
                logger.error("委外入库OFS档案转换", e);
                //失败
            }
        } else {
            logger.info("queryBasicArchives对应returnGoodHeaderDetailsDataDtoList.size为0");
        }
        return consignmachiningInSonDtoArrayList;
    }

    /**
     * 查询委外订单表头对象
     *
     * @author liuyang
     */
    private ScOrderEntity queryScOrder(String ofsPoOrderId) {
        Assert.notNull(ofsPoOrderId, "ofsPoOrderId不能为空!");
        Assert.state(!"".equals(ofsPoOrderId.trim()), "ofsPoOrderId不能为空");

        ScOrderEntity scOrderEntity = new ScOrderEntity();
        scOrderEntity.setVdef20(ofsPoOrderId);
        scOrderEntity.setDr("0");
        scOrderEntity.setDataSourceCode("lets_u8c");
        List<ScOrderEntity> scOrderEntityList = iScOrderDao.query(scOrderEntity);
        if (scOrderEntityList == null || scOrderEntityList.size() == 0) {
            Assert.state(false, "根据O采购订单主键:{}(u委外订单自定义项20存储O采购订单主键)，无法匹配到U8C委外订单", ofsPoOrderId);
        } else if (scOrderEntityList.size() >= 2) {
            Assert.state(false, "根据O采购订单主键:{}(u委外订单自定义项20存储O采购订单主键)，匹配到{}个U8C委外订单", ofsPoOrderId, scOrderEntityList.size());
        }
        return scOrderEntityList.get(0);
    }

    /**
     * 2024年8月25日 14:11:31 查询U8C委外订单明细
     *
     * @param corderid 委外订单表头主键
     * @author liuyang
     */
    private List<ScOrderBEntity> queryScOrderDetail(String corderid) throws Exception {
        Assert.notNull(corderid, "委外订单表头主键不能为空！");
        Assert.state(!"".equals(corderid), "委外订单表头主键不能为空！");

        ScOrderBEntity scOrderBEntity = new ScOrderBEntity();
        scOrderBEntity.setCorderid(corderid);
        scOrderBEntity.setDr("0");
        scOrderBEntity.setDataSourceCode("lets_u8c");
        List<ScOrderBEntity> scOrderBEntityList = iScOrderBDao.query(scOrderBEntity);
        if (scOrderBEntityList == null || scOrderBEntityList.size() == 0) {
            //2024年8月25日 14:16:31 这个验证非常要必要，单据如果没有表体没有行，是很可疑的
            Assert.state(false, "根据U8C委外订单主键({})无法查询到委外订单表体明细行！", corderid);
        }
        return scOrderBEntityList;
    }

    /**
     * 2024年8月6日 10:59:03 查询U8C业务流程
     *
     * @param pkCorp 公司主键
     * @author liuyang
     */
    private BdBusitypeEntity u8cOperationFlow(String pkCorp) throws Exception {
        Assert.notNull(pkCorp, "公司主键不能为空！");
        Assert.state(!"".equals(pkCorp.trim()), "公司主键不能为空！");
        //查询业务流程
        //2024年8月6日 11:33:07 具体的业务流程名称，还需要实施提供
        String processName = "委外加工";
        BdBusitypeEntity bdBusitypeEntity = queryBdBusitypeUtil.queryBdBusitype(processName);
        Assert.notNull(bdBusitypeEntity, "根据业务流程名称({})没有查询到业务流程", processName);
        return bdBusitypeEntity;
    }

    /**
     * 2024年8月20日 15:46:10
     * 生成业务日期，以发货时间作为业务日期
     *
     * @author liuyang
     */
    private String createGenerateBusinessDate(ConsignmachiningInSonDto consignmachiningInSonDto) {
        //TODO 测试
        consignmachiningInSonDto.setShipAt("2024-08-19");
        if (consignmachiningInSonDto != null && consignmachiningInSonDto.getShipAt() != null) {
            String shipAt = consignmachiningInSonDto.getShipAt();
            String businessFormat = null;
            try {
                Date dbill = DateUtil.parse(shipAt);
                businessFormat = DateUtil.format(dbill, "yyyy-MM-dd");
            } catch (Exception e) {
                logger.error("业务日期生成失败", e);
            }
            return businessFormat;
        } else {
            logger.error("生成采购订单入库日期失败，或者shipAt为空! json：{}", JSON.toJSON(consignmachiningInSonDto));
            Assert.state(false, "生成采购订单入库日期失败，或者shipAt为空! json：{}", JSON.toJSON(consignmachiningInSonDto));
            return null;
        }
    }

    /**
     * 匹配U8C采购订单明细行
     * 通过O采购入库单明细行，得到采购订单明细行，最后查找到U8C采购订单明细行
     *
     * @param stockinB           OFS采购入库单明细行
     * @param ofsPoOrderData     OFS采购订单
     * @param scOrderBEntityList U8C委外订单明细行
     * @author liuyang
     */
    private ScOrderBEntity findOfsPoOrderDetailAndU8cPoOrderDetail(StockinOrderSearchResponse.StockinOrder.StockinB stockinB, OfsPoOrderData ofsPoOrderData, List<ScOrderBEntity> scOrderBEntityList) {
        Assert.notNull(stockinB, "stockinB不能为空");
        Assert.notNull(ofsPoOrderData, "ofsPoOrderData不能为空");
        Assert.notNull(scOrderBEntityList, "scOrderBEntityList删除不能为空");

//        OfsPoOrderHeader header = ofsPoOrderData.getHeader();
        List<OfsPoOrderDetails> details = ofsPoOrderData.getDetails();

        //查找采购订单明细行
        OfsPoOrderDetails tarGetOfsPoOrderDetails = null;
        for (int i = 0; i < details.size(); i++) {
            OfsPoOrderDetails ofsPoOrderDetails = details.get(i);
            if (ofsPoOrderDetails.getId().equals(stockinB.getRefOrderDetailId())) {
                tarGetOfsPoOrderDetails = ofsPoOrderDetails;
            }
        }
        Assert.notNull(tarGetOfsPoOrderDetails, "无法匹配到采购订单明细行！ 目标主键：{}", stockinB.getRefOrderDetailId());

        //根据采购订单明细行主键，查找U8C采购订单明细行
        for (int i = 0; i < scOrderBEntityList.size(); i++) {
            ScOrderBEntity scOrderBEntity = scOrderBEntityList.get(i);
            Assert.notNull(scOrderBEntity.getVdef20(), "委外订单明细行v20不能为空(明细行v20存储O采购订单明细行主键)！");
            Assert.state(!"".equals(scOrderBEntity.getVdef20()), "委外订单明细行v20不能为空(明细行v20存储O采购订单明细行主键)！");
            if (tarGetOfsPoOrderDetails.getId().equals(scOrderBEntity.getVdef20())) {
                return scOrderBEntity;
            }
        }
        Assert.state(false, "通过U8C采购订单明细行主键层层匹配到U8C采购明细失败！");
        return null;
    }

    /**
     * 匹配U8C采购订单明细行
     * 通过O采购入库单明细行，得到采购订单明细行，最后查找到U8C采购订单明细行
     *
     * @param stockinB       OFS采购入库单明细行
     * @param ofsPoOrderData OFS采购订单
     * @author liuyang
     */
    private OfsPoOrderDetails findOfsPoOrderDetailRowTraget(StockinOrderSearchResponse.StockinOrder.StockinB stockinB, OfsPoOrderData ofsPoOrderData) {
        Assert.notNull(stockinB, "stockinB不能为空");
        Assert.notNull(ofsPoOrderData, "ofsPoOrderData不能为空");

        List<OfsPoOrderDetails> details = ofsPoOrderData.getDetails();
        for (int i = 0; i < details.size(); i++) {
            OfsPoOrderDetails ofsPoOrderDetails = details.get(i);
            if (ofsPoOrderDetails.getId().equals(stockinB.getRefOrderDetailId())) {
                return ofsPoOrderDetails;
            }
        }
        Assert.state(false, "通过OFS采购入库单，无法匹配到OFS采购订单明细行！ 目标主键：{}", stockinB.getRefOrderDetailId());
        return null;
    }

    /**
     * 查询存货管理档案
     *
     * @param pkCorp   发货公司主键
     * @param stockinB 采购入库单明细行
     * @author liuyang
     */
    private BdInvmandocEntity queryInventoryMan(StockinOrderSearchResponse.StockinOrder.StockinB stockinB, String pkCorp) throws Exception {
        Assert.notNull(stockinB, "sonDetailsDto不能为空");
        Assert.notNull(stockinB.getSkuCode(), "O存货商家编码不能为空");
        Assert.notNull(pkCorp, "发货公司主键不能为空");

        BdInvmandocEntity bdInvmandocEntity = new BdInvmandocEntity();
        bdInvmandocEntity.setInvcode(stockinB.getSkuCode());
        bdInvmandocEntity.setPkCorp(pkCorp);
        List<BdInvmandocEntity> bdInvmandocEntity2 = iBdInvmandocDao.queryBdInvmandocByInvcodeList(bdInvmandocEntity);
        if (bdInvmandocEntity2.size() == 0) {
            Assert.state(false, "根据O商家编码：{} U8C发货公司主键：{}没有查询到存货管理档案", stockinB.getSkuCode(), pkCorp);
        } else if (bdInvmandocEntity2.size() >= 2) {
            Assert.state(false, "根据O商家编码：{} U8C发货公司主键：{}没有查询到存货管理档案", stockinB.getSkuCode(), pkCorp);
        }
        return bdInvmandocEntity2.get(0);
    }

    /**
     * 2024年8月25日 15:12:22
     * 推送U8C委外入库单
     *
     * @param param 原数据json
     * @author liuyang
     */
    public IcGeneralHResultDto sendU8cConsignmachiningIn(String param) throws Exception {
        long startLong = System.currentTimeMillis();
        logger.info("U8C委外入库推送开始，推送参数：" + param + ",U8C_URL:" + ProfilesActiveConstant.U8C_URL);
        String result = HttpRequest.post(ProfilesActiveConstant.U8C_URL).header("appId", "800037")//头信息，多个头信息多次调用此方法即可
                .header("usercode", "admin").header("password", "21232f297a57a5a743894a0e4a801fc3").header("system", "lz").header("trantype", "PK").header("apiCode", "8000370056")//头信息，多个头信息多次调用此方法即可
                .header("publicKey", "ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj")//头信息，多个头信息多次调用此方法即可
                .header("secretKey", "fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")//头信息，多个头信息多次调用此方法即可
                .body(param)//表单内容
                .timeout(20000)//超时，毫秒
                .execute().body();
        logger.info("U8C委外入库推送结束，返回参数：" + result);
        long endLong = System.currentTimeMillis();
        logger.info("U8C委外入库推送耗时：" + (endLong - startLong));

        if (result == null) {
            result = "";
        }

        JSONObject jsonObject = JSON.parseObject(result);
        result = String.valueOf(jsonObject.get("attribute"));

        boolean isSuccess = false;
        IcGeneralHResultDto icGeneralHResultDto = null;
        if (result != null && !"".equals(result)) {
            ReusltStrDto reusltStrDto = JSON.parseObject(result, ReusltStrDto.class);
            if ("success".equals(reusltStrDto.getStatus())) {
                icGeneralHResultDto = resultDataHandle(reusltStrDto.getData());
                isSuccess = true;
            }
        }
        if (!isSuccess) {
            Assert.state(false, "O采购入库业务推送U8C委外入库失败 接口返回结果：{}", result);
        }
        return icGeneralHResultDto;
    }

    /**
     * 返回结果解析处理，在确认了success后调用
     *
     * @author liuyang
     */
    private IcGeneralHResultDto resultDataHandle(String resultData) {
        try {
            if (resultData != null && !"".equals(resultData)) {
                if (resultData.contains("[")) {
                    resultData = resultData.substring(1, resultData.length() - 1);
                }
                return JSON.parseObject(resultData, IcGeneralHResultDto.class);
            }
        } catch (Exception e) {
            logger.error("resultDataHandle方法解析返回参数失败的错误", e);
            //如果解析失败，记录原因，但是不能影响结果的记录
        }
        return null;
    }
}